#!/bin/python
# -*- coding: utf-8 -*-
"""
help_screen.py
Lines to be displayed on the help screen.

Author: GrimAndGreedy
License: MIT
"""

help_lines = [
    ['Navigation:',                     ''],
    ['    Up/Down and k/j',              'Move up/down'],
    ['    K and J',                         'Move up/down by five'],
    ['    Ctrl+f and Ctrl+b',                         'Page up/down'],
    ['    u and d',                         'Half page up/down'],
    ['    g/home',                   'Go to top of page'],
    ['    G/end',                    'Go to bottom of page'],
    ['',                             ''],
    ['Selection:',                      ''],
    ['    Space',                       'Toggle selection'],
    ['    m or ^a',                    'Select all'],
    ['    M or ^r',                    'Deselect all'],
    ['    v',                           'Start and stop visual selection'],
    ['    V',                           'Start and stop visual deselection'],
    ['    l/Enter',                     'Submit selection'],
    ['',                             ''],
    ['Modes:',                           ''],
    ['    Tab',                           'Cycle between modes forwards'],
    ['    Shift+Tab',                           'Cycle between modes backwards'],
    ['',                             ''],
    ['Sorting:',                        ''],
    ['    0-9',                         'Select column (1-based index)'],
    ['    </>',                         'Focus on next/previous column'],
    ['    s',                           'Cycle sort method'],
    ['    t',                           'Toggle sort order (asc. or desc.)'],
    ['',                             ''],
    ['Filter or search rows (regexp):',                    ''],
    ['    f',                           'Begin filtering rows'],
    ['    /',                           'Begin searching rows'],
    ['      --[1-9]',                   'Specify column'],
    ['      --i',                       'Make match case sensitive'],
    ['      --v',                       'Invert filter'],
    ['    n/i',                       'Continue search forwards'],
    ['    N/I',                       'Continue search backwards'],
    ['',                             ''],
    ['Pipe',                    ''],
    ['    |',                           'Pipe selected rows from active column into a specified command'],
    ['    e.g.,',                           'xargs -d \n -I{} echo {} > ~/rubbish.txt'],
    ['    e.g.,',                           'xargs -d \n -I{} mpv {}'],
    ['',                             ''],

    ['Load and Save',                    ''],
    ['    Ctrl+s',                           'Open save dialog. Then pick output type and location to save.'],
    ['    Ctrl+o',                           'Open previously saved data.'],
    ['',                             ''],
    ['Columns:',                         ''],
    ['    0-9',                         'Select column (1-based index)'],
    ['    </>',                         'Focus on next/previous column'],
    ['    Shift+1-9',                   'Hide/show column'],
    ['    [/]',                         'Increase/Decrease column width'],
    # ['    {',                           'Move focused column left'],
    # ['    }',                           'Move focused column right'],
    ['',                             ''],
    ['Clipboard:',                      ''],
    ['    y',                           'Open dialogue to select how you want the selected rows to be copied'],
    ['',                             ''],
    ['Settings:',                           ''],
    ['    ~',                           'Open settings picker'],
    ['    `',                           'settings input'],
    ['        !\\d+',                       'Show/hide column'],
    ['        !r',                          'toggle auto refresh'],
    ['    \\',                          'Clear settings'],
    ['',                             ''],
    ['Input fields:',                           ''],
    ['    Ctrl+r',                           'Insert current cell (cursor, sort_col) into input bar'],
    ['    Ctrl+e',                           'End'],
    ['    Ctrl+a',                           'Start'],
    ['    Ctrl+u',                           'Delete to start'],
    ['    Ctrl+k',                           'Delete to end'],
    ['    Ctrl+f/Ctrl+b',                           'Forwards/Backwards one character.'],
    ['Misc:',                           ''],
    ['    ?',                           'Show this help screen'],
    ['    _',                           'Quick toggle footer'],
    ['    O',                           'Try to open the data in the focused column of the selected rows as file paths.'],
    ['        ',                           'You may need to change the cwd by pressing ` and then entering e.g., "cwd=~/Pictures"'],
    ['    +',                           'Increase lines per page'],
    ['    -',                           'Decrease lines per page'],
    ['    :',                           'Specify user optios to be returned with selection'],
    ['    o',                           'Open submenu to specify option'],
    ['    q',                           'Exit menu with no selection.'],
    ['    ?',                           'Show this help page'],
    ['    Ctrl+c',                           'Immediate quit.'],
    ['    Ctrl+l',                           'Redraw screen.'],
    ['    .',                           'Redo setting'],
    ['',                             ''],
]



#!/bin/python
# -*- coding: utf-8 -*-
"""
pane_functions.py
Functions which are run by a listpick Picker to display data in a pane.

Author: GrimAndGreedy
License: MIT
"""

import curses
import os
from listpick.pane.pane_utils import get_file_attributes, get_graph_string, escape_ansi
from listpick.pane.get_data import update_file_attributes

def left_start_pane(stdscr, x, y, w, h, state, row, cell, data: list = [], test: bool = False):
    """
    Display file attributes in right pane.
    """
    if test: return True

    # Title
    for i in range(h):
        s = '*'*w
        stdscr.addstr(y+i, x, s)


    stdscr.addstr(y, x, "+")
    stdscr.addstr(y+h-1, x, "+")
    stdscr.addstr(y, x+w-1, "+")
    stdscr.addstr(y+h-1, x+w-1, "+")


    stdscr.addstr(y+1, x, f"{w},{h}")

    return []


def left_split_file_attributes(stdscr, x, y, w, h, state, row, cell, data: list = [], test: bool = False):
    """
    Display file attributes in right pane.
    """
    if test: return True

    # Title
    title = "File attributes"
    if len(title) < w: title = f"{title:^{w}}"
    stdscr.addstr(y, x,title[:w], curses.color_pair(state["colours_start"]+4) | curses.A_BOLD)

    # Separator
    for j in range(h):
        stdscr.addstr(j+y, x+w-1, ' ', curses.color_pair(state["colours_start"]+16))

    # Display pane count
    pane_count = len(state["right_panes"])
    pane_index = state["right_pane_index"]
    if pane_count > 1:
        s = f" {pane_index+1}/{pane_count} "
        stdscr.addstr(y+h-1, x, s, curses.color_pair(state["colours_start"]+20))

    # Filename/cursor cell value
    stdscr.addstr(y+2, x+2, cell[:w-3])

    attributes = get_file_attributes(cell)
    for i, attr in enumerate(attributes):
        stdscr.addstr(y+3+i, x+4, attr[:w-5])

    return []


def left_split_file_attributes_dynamic(stdscr, x, y, w, h, state, row, cell, data: list = [], test: bool = False):
    """
    Display file attributes in right pane.
    """
    if test: return True

    # Title
    title = "File attributes"
    if len(title) < w: title = f"{title:^{w}}"
    stdscr.addstr(y, x,title[:w], curses.color_pair(state["colours_start"]+4) | curses.A_BOLD)

    # Separator
    for j in range(h):
        stdscr.addstr(j+y, x+w-1, ' ', curses.color_pair(state["colours_start"]+16))

    # Display pane count
    pane_count = len(state["right_panes"])
    pane_index = state["right_pane_index"]
    if pane_count > 1:
        s = f" {pane_index+1}/{pane_count} "
        stdscr.addstr(y+h-1, x, s, curses.color_pair(state["colours_start"]+20))

    if len(state["indexed_items"]) == 0:
        return []

    # Filename/cursor cell value
    stdscr.addstr(y+2, x+2, cell[:w-3])

    # If the cursor-hovered file is different then reload the data
    if data[1] != cell:
        data[:] = update_file_attributes(data, state)

    # attributes = get_file_attributes(cell)
    if len(data)  == 0: return []
    attributes = data[0]
    for i, attr in enumerate(attributes):
        stdscr.addstr(y+3+i, x+4, attr[:w-5])

    return []

def left_split_graph(stdscr, x, y, w, h, state, row, cell, data: list = [], test: bool = False):
    """
    Display a graph of the data in right pane.

    data[0] = x_vals
    data[1] = y_vals
    data[2] = id
    """
    if test: return True

    # Title
    title = "Graph"
    if len(title) < w: title = f"{title:^{w}}"
    stdscr.addstr(y, x,title[:w], curses.color_pair(state["colours_start"]+4) | curses.A_BOLD)

    # Separator
    for j in range(h):
        stdscr.addstr(j+y, x+w-1, ' ', curses.color_pair(state["colours_start"]+16))


    # Display pane count
    pane_count = len(state["right_panes"])
    pane_index = state["right_pane_index"]
    if pane_count > 1:
        s = f" {pane_index+1}/{pane_count} "
        stdscr.addstr(y+h-1, x, s, curses.color_pair(state["colours_start"]+20))

    try:
        import plotille as plt
    except:
        s = f"No module named 'plotille'"
        stdscr.addstr(y+2, x+1, s[:w-2])
        return None



    # x_vals, y_vals = list(range(100)), [x**2 for x in range(100)]
    if data in [[], {}, None]:
        return None
    x_vals, y_vals = data[0], data[1]
    graph_str = get_graph_string(x_vals, y_vals, width=w-3-10, height=h-3)
    for i, s in enumerate(graph_str.split("\n")):
        s = escape_ansi(s)
        stdscr.addstr(y+2+i, x+1, s[:w-2])

    return []




def left_split_display_list(stdscr, x, y, w, h, state, row, cell, data: list = [], test: bool = False):
    """
    data[0]:str = title
    data[1]:list[str] = list of strings to display
    """
    if test: return True

    # Title
    title = data[0]
    if len(title) < w: title = f"{title:^{w}}"
    stdscr.addstr(y, x,title[:w], curses.color_pair(state["colours_start"]+4) | curses.A_BOLD)

    # Separator
    for j in range(h):
        stdscr.addstr(j+y, x+w-1, ' ', curses.color_pair(state["colours_start"]+16))


    # Display pane count
    pane_count = len(state["right_panes"])
    pane_index = state["right_pane_index"]
    if pane_count > 1:
        s = f" {pane_index+1}/{pane_count} "
        stdscr.addstr(y+h-1, x, s, curses.color_pair(state["colours_start"]+20))

    if data in [[], {}, None]:
        return None

    items = data[1]
    number_to_display = min(len(items), h-3)
    for i in range(number_to_display):
        s = items[i]
        stdscr.addstr(y+1+i, x+2, s[:w-2])

    if number_to_display < len(items):
        stdscr.addstr(y+1+number_to_display, x+2, f" ... {len(items)-number_to_display} more"[:w-2])


    return []



#!/bin/python
# -*- coding: utf-8 -*-
"""
picker_colours.py
Define colour options for listpick--default, help, and notification colours.

Author: GrimAndGreedy
License: MIT
"""

import curses
from typing import Dict
import logging

logger = logging.getLogger('picker_log')


def get_colours(pick:int=0) -> Dict[str, int]:
    """ Define colour options for listpick. """
    logger.info("function get_colours (picker_colours.py)")
    colours = [
        ### (0) Green header, green title, green modes, purple selected, blue cursor
    {
        'background': 232,
        'normal_fg': 255,
        'unselected_bg': 232,
        'unselected_fg': 255,
        'cursor_bg': 21,
        'cursor_fg': 255,
        'selected_bg': 54,
        'selected_fg': 255,
        'header_bg': 255,
        'header_fg': 232,
        'error_bg': 232,
        'error_fg': curses.COLOR_RED,
        'complete_bg': 232,
        'complete_fg': 82,
        'waiting_bg': 232,
        'waiting_fg': curses.COLOR_YELLOW,
        'active_bg': 232,
        'active_fg': 33,
        'paused_bg': 232,
        'paused_fg': 244,
        'search_bg': 162,
        'search_fg': 255,
        'active_input_bg': 255,
        'active_input_fg': 28,
        'modes_selected_bg': 28,
        'modes_selected_fg': 255,
        'modes_unselected_bg': 255,
        'modes_unselected_fg': 232,
        'title_bar': 28,
        'title_bg': 28,
        'title_fg': 255,
        'scroll_bar_bg': 247,
        'selected_header_column_bg': 232,
        'selected_header_column_fg': 255,
        'unselected_header_column_bg': 255,
        'unselected_header_column_fg': 232,
        'footer_bg': 28,
        'footer_fg': 255,
        'refreshing_bg': 28,
        'refreshing_fg': 255,
        'refreshing_inactive_bg': 28,
        'refreshing_inactive_fg': 232,
        '40pc_bg': 232,
        '40pc_fg': 166,
        'footer_string_bg': 28,
        'footer_string_fg': 255,
        'selected_cell_bg': 54,
        'selected_cell_fg': 255,
        'deselecting_cell_bg': 162,
        'deselecting_cell_fg': 255,
        'active_column_bg': 234,
        'active_column_fg': 255,
    },
        ### (1) Black and white
    {
        'background': 232,
        'normal_fg': 255,
        'unselected_bg': 232,
        'unselected_fg': 255,
        'cursor_bg': 255,
        'cursor_fg': 232,
        'selected_bg': 255,
        'selected_fg': 232,
        'header_bg': 255,
        'header_fg': 232,
        'error_bg': 232,
        'error_fg': 255,
        'complete_bg': 232,
        'complete_fg': 255,
        'waiting_bg': 232,
        'waiting_fg': 255,
        'active_bg': 232,
        'active_fg': 255,
        'paused_bg': 232,
        'paused_fg': 255,
        'search_bg': 255,
        'search_fg': 232,
        'active_input_bg': 232,
        'active_input_fg': 255,
        'modes_selected_bg': 255,
        'modes_selected_fg': 232,
        'modes_unselected_bg': 232,
        'modes_unselected_fg': 255,
        'title_bar': 232,
        'title_bg': 255,
        'title_fg': 232,
        'scroll_bar_bg': 255,
        'selected_header_column_bg': 232,
        'selected_header_column_fg': 255,
        'unselected_header_column_bg': 255,
        'unselected_header_column_fg': 232,
        'footer_bg': 255,
        'footer_fg': 232,
        'refreshing_bg': 255,
        'refreshing_fg': 232,
        'refreshing_inactive_bg': 232,
        'refreshing_inactive_fg': 255,
        '40pc_bg': 232,
        '40pc_fg': 255,
        'footer_string_bg': 255,
        'footer_string_fg': 232,
        'selected_cell_bg': 237,
        'selected_cell_fg': 255,
        'deselecting_cell_bg': 162,
        'deselecting_cell_fg': 255,
        'active_column_bg': 234,
        'active_column_fg': 255,
    },
        ### (2) Blue header, blue title, blue modes, purple selected, green cursor
    {
        'background': 232,
        'normal_fg': 255,
        'unselected_bg': 232,
        'unselected_fg': 255,
        'cursor_bg': 28,
        'cursor_fg': 255,
        'selected_bg': 135,
        'selected_fg': 255,
        'header_bg': 255,
        'header_fg': 232,
        'error_bg': 232,
        'error_fg': curses.COLOR_RED,
        'complete_bg': 232,
        'complete_fg': 82,
        'waiting_bg': 232,
        'waiting_fg': curses.COLOR_YELLOW,
        'active_bg': 232,
        'active_fg': 33,
        'paused_bg': 232,
        'paused_fg': 244,
        'search_bg': 162,
        'search_fg': 255,
        'active_input_bg': 255,
        'active_input_fg': 25,
        'modes_selected_bg': 25,
        'modes_selected_fg': 255,
        'modes_unselected_bg': 255,
        'modes_unselected_fg': 232,
        'title_bar': 25,
        'title_bg': 25,
        'title_fg': 255,
        'scroll_bar_bg': 247,
        'selected_header_column_bg': 232,
        'selected_header_column_fg': 255,
        'unselected_header_column_bg': 255,
        'unselected_header_column_fg': 232,
        'footer_bg': 25,
        'footer_fg': 255,
        'refreshing_bg': 25,
        'refreshing_fg': 255,
        'refreshing_inactive_bg': 25,
        'refreshing_inactive_fg': 232,
        '40pc_bg': 232,
        '40pc_fg': 166,
        'footer_string_bg': 25,
        'footer_string_fg': 255,
        'selected_cell_bg': 237,
        'selected_cell_fg': 255,
        'deselecting_cell_bg': 162,
        'deselecting_cell_fg': 255,
        'active_column_bg': 234,
        'active_column_fg': 255,
    },
        ### (3) Purple header, purple title, white modes, green selected, blue cursor
    {
        'background': 232,
        'normal_fg': 255,
        'unselected_bg': 232,
        'unselected_fg': 255,
        'cursor_bg': 54,
        'cursor_fg': 255,
        'selected_bg': 135,
        'selected_fg': 255,
        'header_bg': 255,
        'header_fg': 232,
        'error_bg': 232,
        'error_fg': curses.COLOR_RED,
        'complete_bg': 232,
        'complete_fg': 82,
        'waiting_bg': 232,
        'waiting_fg': curses.COLOR_YELLOW,
        'active_bg': 232,
        'active_fg': 33,
        'paused_bg': 232,
        'paused_fg': 244,
        'search_bg': 162,
        'search_fg': 255,
        'active_input_bg': 255,
        'active_input_fg': 57,
        'modes_selected_bg': 57,
        'modes_selected_fg': 255,
        'modes_unselected_bg': 255,
        'modes_unselected_fg': 232,
        'title_bar': 57,
        'title_bg': 57,
        'title_fg': 255,
        'scroll_bar_bg': 247,
        'selected_header_column_bg': 232,
        'selected_header_column_fg': 255,
        'unselected_header_column_bg': 255,
        'unselected_header_column_fg': 232,
        'footer_bg': 57,
        'footer_fg': 255,
        'refreshing_bg': 57,
        'refreshing_fg': 255,
        'refreshing_inactive_bg': 57,
        'refreshing_inactive_fg': 232,
        '40pc_bg': 232,
        '40pc_fg': 166,
        'footer_string_bg': 57,
        'footer_string_fg': 255,
        'selected_cell_bg': 135,
        'selected_cell_fg': 232,
        'deselecting_cell_bg': 162,
        'deselecting_cell_fg': 255,
        'active_column_bg': 234,
        'active_column_fg': 255,
    },
        ## (4) 3-bit colours
    {
        'background': curses.COLOR_BLACK,
        'normal_fg': curses.COLOR_WHITE,
        'unselected_bg': curses.COLOR_BLACK,
        'unselected_fg': curses.COLOR_WHITE,
        'cursor_bg': curses.COLOR_BLUE,
        'cursor_fg': curses.COLOR_BLACK,
        'selected_bg': curses.COLOR_MAGENTA,
        'selected_fg': curses.COLOR_BLACK,
        'header_bg': curses.COLOR_WHITE,
        'header_fg': curses.COLOR_BLACK,
        'error_bg': curses.COLOR_BLACK,
        'error_fg': curses.COLOR_RED,
        'complete_bg': curses.COLOR_BLACK,
        'complete_fg': curses.COLOR_GREEN,
        'waiting_bg': curses.COLOR_BLACK,
        'waiting_fg': curses.COLOR_YELLOW,
        'active_bg': curses.COLOR_BLACK,
        'active_fg': curses.COLOR_BLUE,
        'paused_bg': curses.COLOR_BLACK,
        'paused_fg': curses.COLOR_WHITE,
        'search_bg': curses.COLOR_WHITE,
        'search_fg': curses.COLOR_MAGENTA,
        'active_input_bg': curses.COLOR_BLACK,
        'active_input_fg': curses.COLOR_WHITE,
        'modes_selected_bg': curses.COLOR_GREEN,
        'modes_selected_fg': curses.COLOR_BLACK,
        'modes_unselected_bg': curses.COLOR_WHITE,
        'modes_unselected_fg': curses.COLOR_BLACK,
        'title_bar': curses.COLOR_BLACK,
        'title_bg': curses.COLOR_GREEN,
        'title_fg': curses.COLOR_BLACK,
        'scroll_bar_bg': curses.COLOR_WHITE,
        'selected_header_column_bg': curses.COLOR_BLACK,
        'selected_header_column_fg': curses.COLOR_WHITE,
        'unselected_header_column_bg': curses.COLOR_WHITE,
        'unselected_header_column_fg': curses.COLOR_BLACK,
        'footer_bg': curses.COLOR_GREEN,
        'footer_fg': curses.COLOR_BLACK,
        'refreshing_bg': curses.COLOR_GREEN,
        'refreshing_fg': curses.COLOR_BLACK,
        'refreshing_inactive_bg': curses.COLOR_GREEN,
        'refreshing_inactive_fg': curses.COLOR_WHITE,
        '40pc_bg': curses.COLOR_BLACK,
        '40pc_fg': curses.COLOR_YELLOW,
        'footer_string_bg': curses.COLOR_GREEN,
        'footer_string_fg': curses.COLOR_BLACK,
        'selected_cell_bg': 237,
        'selected_cell_fg': 255,
        'deselecting_cell_bg': curses.COLOR_MAGENTA,
        'deselecting_cell_fg': curses.COLOR_WHITE,
        'active_column_bg': curses.COLOR_BLACK,
        'active_column_fg': curses.COLOR_WHITE,
    },
    ]
    for colour in colours:
        colour["20pc_bg"] = colour["background"]
        colour["40pc_bg"] = colour["background"]
        colour["60pc_bg"] = colour["background"]
        colour["80pc_bg"] = colour["background"]
        colour["100pc_bg"] = colour["background"]
        colour["error_bg"] = colour["background"]
        colour["complete_bg"] = colour["background"]
        colour["waiting_bg"] = colour["background"]
        colour["active_bg"] = colour["background"]
        colour["paused_bg"] = colour["background"]
        colour["paused_bg"] = colour["background"]
        # colour["search_bg"] = colour["background"]
    if pick > len(colours) - 1:
        return colours[0]
    return colours[pick]

def get_help_colours(pick: int=0) -> Dict[str, int]:
    """ Define help colour options for listpick. """
    logger.info("function get_help_colours (picker_colours.py)")
    colours = get_colours(pick)
    # colours = [get_colours(i) for i in range(get_theme_count())]
    # for i in range(len(colours)):

    # 3-bit colours
    colours_3_bit = [4]
    if pick in colours_3_bit:
        colours['cursor_bg'] = curses.COLOR_WHITE
        colours['cursor_fg'] = curses.COLOR_BLACK
        colours['selected_bg'] = curses.COLOR_WHITE
        colours['selected_fg'] = curses.COLOR_BLACK
    else:
        colours['cursor_bg'] = 235
        colours['cursor_fg'] = 255
        colours['selected_bg'] = 25
        colours['selected_fg'] = 255

    return colours


def get_notification_colours(pick:int=0) -> Dict[str, int]:
    """ Define notification colour options for listpick. """
    logger.info("function get_notification_colours (picker_colours.py)")
    colours = get_colours(pick)

    # Black and white
    if pick == 1:
        colours['background'] = 237
        colours['unselected_bg'] = 237
        colours['cursor_bg'] = 237
        colours['cursor_fg'] = 255
        colours['selected_bg'] = 237
        colours['selected_fg'] = 255

    # 3-bit colours
    elif pick == 4:
        colours['background'] = curses.COLOR_BLACK
        colours['unselected_bg'] = curses.COLOR_BLACK
        colours['cursor_bg'] = curses.COLOR_WHITE
        colours['cursor_fg'] = curses.COLOR_BLACK
        colours['selected_bg'] = curses.COLOR_WHITE
        colours['selected_fg'] = curses.COLOR_BLACK
    else:
        colours['background'] = 237
        colours['unselected_bg'] = 237
        colours['cursor_bg'] = 238
        colours['cursor_fg'] = 255
        colours['selected_bg'] = 237

    colours['selected_header_column_fg'] = colours['header_fg']
    colours['selected_header_column_bg'] = colours['header_bg']
    colours['unselected_header_column_fg'] = colours['header_fg']
    colours['unselected_header_column_bg'] = colours['header_bg']

    return colours

def get_fallback_colours() -> Dict[str, int]:
    logger.info("function get_fallback_colours (picker_colours.py)")
    return get_colours(4)

def get_theme_count() -> int:
    """ Get the number of themes. """
    logger.info("function get_theme_count (picker_colours.py)")
    col_list = []
    i = 0
    for i in range(100):
        x = get_colours(i)
        if x in col_list:
            break
        col_list.append(x)
    return i

def check_for_missing_keys():
    """ Check if any of the colour_dicts have missing keys--i.e., if their keys are incongruous. """
    ref = 0
    count = get_theme_count()

    colours = [get_colours(i) for i in range(count)]

    diffs = []
    for i in range(count):
        print(f"{i}")
        for j in range(count):
            diff = set(colours[i].keys()) - set(colours[j].keys())
            print(f"    colours[{i}] - colours[{j}] = {diff}")
            if diff:
                diffs.append(f"    colours[{i}] - colours[{j}] = {diff}")

    if diffs:
        print("*"*80)
        print("*"*80)
        for d in diffs:
            print(d)




# Set a base value to ensure that we have no collisions with any unicode characters
BASE_VALUE = 2**32

# Define constants for alt+a to alt+z
META_A = BASE_VALUE + 1000
META_B = BASE_VALUE + 1001
META_C = BASE_VALUE + 1002
META_D = BASE_VALUE + 1003
META_E = BASE_VALUE + 1004
META_F = BASE_VALUE + 1005
META_G = BASE_VALUE + 1006
META_H = BASE_VALUE + 1007
META_I = BASE_VALUE + 1008
META_J = BASE_VALUE + 1009
META_K = BASE_VALUE + 1010
META_L = BASE_VALUE + 1011
META_M = BASE_VALUE + 1012
META_N = BASE_VALUE + 1013
META_O = BASE_VALUE + 1014
META_P = BASE_VALUE + 1015
META_Q = BASE_VALUE + 1016
META_R = BASE_VALUE + 1017
META_S = BASE_VALUE + 1018
META_T = BASE_VALUE + 1019
META_U = BASE_VALUE + 1020
META_V = BASE_VALUE + 1021
META_W = BASE_VALUE + 1022
META_X = BASE_VALUE + 1023
META_Y = BASE_VALUE + 1024
META_Z = BASE_VALUE + 1025

# Define constants for alt+A to alt+Z (using uppercase)
META_a = BASE_VALUE + 1050
META_b = BASE_VALUE + 1051
META_c = BASE_VALUE + 1052
META_d = BASE_VALUE + 1053
META_e = BASE_VALUE + 1054
META_f = BASE_VALUE + 1055
META_g = BASE_VALUE + 1056
META_h = BASE_VALUE + 1057
META_i = BASE_VALUE + 1058
META_j = BASE_VALUE + 1059
META_k = BASE_VALUE + 1060
META_l = BASE_VALUE + 1061
META_m = BASE_VALUE + 1062
META_n = BASE_VALUE + 1063
META_o = BASE_VALUE + 1064
META_p = BASE_VALUE + 1065
META_q = BASE_VALUE + 1066
META_r = BASE_VALUE + 1067
META_s = BASE_VALUE + 1068
META_t = BASE_VALUE + 1069
META_u = BASE_VALUE + 1070
META_v = BASE_VALUE + 1071
META_w = BASE_VALUE + 1072
META_x = BASE_VALUE + 1073
META_y = BASE_VALUE + 1074
META_z = BASE_VALUE + 1075


META_0 = BASE_VALUE + 1090
META_1 = BASE_VALUE + 1091
META_2 = BASE_VALUE + 1092
META_3 = BASE_VALUE + 1093
META_4 = BASE_VALUE + 1094
META_5 = BASE_VALUE + 1095
META_6 = BASE_VALUE + 1096
META_7 = BASE_VALUE + 1097
META_8 = BASE_VALUE + 1098
META_9 = BASE_VALUE + 1099


META_BS = BASE_VALUE + 1100

# Dictionary to map character to its constant value
META_KEY_MAP = {
    'a': META_a, 'b': META_b, 'c': META_c, 'd': META_d, 'e': META_e,
    'f': META_f, 'g': META_g, 'h': META_h, 'i': META_i, 'j': META_j,
    'k': META_k, 'l': META_l, 'm': META_m, 'n': META_n, 'o': META_o,
    'p': META_p, 'q': META_q, 'r': META_r, 's': META_s, 't': META_t,
    'u': META_u, 'v': META_v, 'w': META_w, 'x': META_x, 'y': META_y,
    'z': META_z, 'A': META_A, 'B': META_B, 'C': META_C, 'D': META_D,
    'E': META_E, 'F': META_F, 'G': META_G, 'H': META_H, 'I': META_I,
    'J': META_J, 'K': META_K, 'L': META_L, 'M': META_M, 'N': META_N,
    'O': META_O, 'P': META_P, 'Q': META_Q, 'R': META_R, 'S': META_S,
    'T': META_T, 'U': META_U, 'V': META_V, 'W': META_W, 'X': META_X,
    'Y': META_Y, 'Z': META_Z
}
